'use strict';
/**
 * Created by andy on 14-12-25.
 */
/**
 *
 *  Secure Hash Algorithm (SHA1)
 *  http://www.webtoolkit.info/
 *
 **/

function SHA1(msg) {

    function rotate_left(n, s) {
        var t4 = ( n << s ) | (n >>> (32 - s));
        return t4;
    };

    function lsb_hex(val) {
        var str = "";
        var i;
        var vh;
        var vl;

        for (i = 0; i <= 6; i += 2) {
            vh = (val >>> (i * 4 + 4)) & 0x0f;
            vl = (val >>> (i * 4)) & 0x0f;
            str += vh.toString(16) + vl.toString(16);
        }
        return str;
    };

    function cvt_hex(val) {
        var str = "";
        var i;
        var v;

        for (i = 7; i >= 0; i--) {
            v = (val >>> (i * 4)) & 0x0f;
            str += v.toString(16);
        }
        return str;
    };


    function Utf8Encode(string) {
        string = string.replace(/\r\n/g, "\n");
        var utftext = "";

        for (var n = 0; n < string.length; n++) {

            var c = string.charCodeAt(n);

            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if ((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }

        }

        return utftext;
    };

    var blockstart;
    var i, j;
    var W = new Array(80);
    var H0 = 0x67452301;
    var H1 = 0xEFCDAB89;
    var H2 = 0x98BADCFE;
    var H3 = 0x10325476;
    var H4 = 0xC3D2E1F0;
    var A, B, C, D, E;
    var temp;

    msg = Utf8Encode(msg);

    var msg_len = msg.length;

    var word_array = new Array();
    for (i = 0; i < msg_len - 3; i += 4) {
        j = msg.charCodeAt(i) << 24 | msg.charCodeAt(i + 1) << 16 |
            msg.charCodeAt(i + 2) << 8 | msg.charCodeAt(i + 3);
        word_array.push(j);
    }

    switch (msg_len % 4) {
        case 0:
            i = 0x080000000;
            break;
        case 1:
            i = msg.charCodeAt(msg_len - 1) << 24 | 0x0800000;
            break;

        case 2:
            i = msg.charCodeAt(msg_len - 2) << 24 | msg.charCodeAt(msg_len - 1) << 16 | 0x08000;
            break;

        case 3:
            i = msg.charCodeAt(msg_len - 3) << 24 | msg.charCodeAt(msg_len - 2) << 16 | msg.charCodeAt(msg_len - 1) << 8 | 0x80;
            break;
    }

    word_array.push(i);

    while ((word_array.length % 16) != 14) word_array.push(0);

    word_array.push(msg_len >>> 29);
    word_array.push((msg_len << 3) & 0x0ffffffff);


    for (blockstart = 0; blockstart < word_array.length; blockstart += 16) {

        for (i = 0; i < 16; i++) W[i] = word_array[blockstart + i];
        for (i = 16; i <= 79; i++) W[i] = rotate_left(W[i - 3] ^ W[i - 8] ^ W[i - 14] ^ W[i - 16], 1);

        A = H0;
        B = H1;
        C = H2;
        D = H3;
        E = H4;

        for (i = 0; i <= 19; i++) {
            temp = (rotate_left(A, 5) + ((B & C) | (~B & D)) + E + W[i] + 0x5A827999) & 0x0ffffffff;
            E = D;
            D = C;
            C = rotate_left(B, 30);
            B = A;
            A = temp;
        }

        for (i = 20; i <= 39; i++) {
            temp = (rotate_left(A, 5) + (B ^ C ^ D) + E + W[i] + 0x6ED9EBA1) & 0x0ffffffff;
            E = D;
            D = C;
            C = rotate_left(B, 30);
            B = A;
            A = temp;
        }

        for (i = 40; i <= 59; i++) {
            temp = (rotate_left(A, 5) + ((B & C) | (B & D) | (C & D)) + E + W[i] + 0x8F1BBCDC) & 0x0ffffffff;
            E = D;
            D = C;
            C = rotate_left(B, 30);
            B = A;
            A = temp;
        }

        for (i = 60; i <= 79; i++) {
            temp = (rotate_left(A, 5) + (B ^ C ^ D) + E + W[i] + 0xCA62C1D6) & 0x0ffffffff;
            E = D;
            D = C;
            C = rotate_left(B, 30);
            B = A;
            A = temp;
        }

        H0 = (H0 + A) & 0x0ffffffff;
        H1 = (H1 + B) & 0x0ffffffff;
        H2 = (H2 + C) & 0x0ffffffff;
        H3 = (H3 + D) & 0x0ffffffff;
        H4 = (H4 + E) & 0x0ffffffff;

    }

    var temp = cvt_hex(H0) + cvt_hex(H1) + cvt_hex(H2) + cvt_hex(H3) + cvt_hex(H4);

    return temp.toLowerCase();

}

'use strict';
function copy(obj) {
    if (obj == null || typeof (obj) != 'object')
        return obj;

    var temp = obj.constructor();
    // changed

    for (var key in obj) {
        if (obj.hasOwnProperty(key)) {
            temp[key] = copy(obj[key]);
        }
    }
    return temp;
}

function isType(type) {
    return function (obj) {
        return {}.toString.call(obj) == "[object " + type + "]"
    }
}

var isFunction = isType("Function");

function Resource(appId, appKey, baseurl) {
    var now = Date.now();
    this.appId = appId;
    this.baseurl = baseurl || "https://d.apicloud.com/mcm/api";
    this.appCode = SHA1(appId + "UZ" + appKey + "UZ" + now) + "." + now;
    this.defaultactions = {
        'get': { method: 'GET', params: ["_id", "_relation"] }, //_relationid 后续支持
        'save': { method: 'POST', params: ["_id", "_relation"] }, //_relationid 后续支持
        'query': { method: 'GET', params: ["filter"] },
        'delete': { method: 'DELETE', params: ["_id", "_relation"] }, //_relationid 后续支持
        'login': { method: "POST", params: ["username", "password"] },
        'logout': { method: "POST" },
        'count': { method: "GET", params: ["_id", "_relation", "filter"] },
        'exists': { method: "GET", params: ["_id"] },
        'findOne': { method: 'GET', params: ["filter"] },
        'verify': { method: "POST", params: ["email", "language", "username"], alias: "verifyEmail" },
        'reset': { method: "POST", params: ["email", "language", "username"], alias: "resetRequest" }
    };
    this.headers = {};
    this.setHeaders("X-APICloud-AppId", this.appId);
    this.setHeaders("X-APICloud-AppKey", this.appCode);
    this.setHeaders("Content-Type", "application/json;");
}
Resource.prototype.setHeaders = function (key, value) {
    this.headers[key] = value;
}
Resource.prototype.batch = function (requests, callback) {
    for (var i = 0, len = requests.length; i < len; i++) {
        var request = requests[i];
        if (request["method"] && request["method"].toUpperCase() === "GET" && request["body"]&&request["body"]["filter"]) {
            var url = request["path"];
            var index = url.indexOf('?');
            request["path"] = url.substring(0, index) + "?filter=" + JSON.stringify(request["body"]["filter"]);
            delete request["body"];
        }
    }
    var ajaxConfig = {
        url: this.baseurl + "/batch",
        method: "POST",
        data: {
            body: JSON.stringify({ requests: requests })
        }
    }
    ajaxConfig["headers"] = {};
    for (var header in this.headers) {
        ajaxConfig["headers"][header] = this.headers[header];
    }
    api.ajax(ajaxConfig, function (ret, err) {
        callback(ret, err)
    });
}
Resource.prototype.upload = function (modelName, isFilter, item, params, callback) {
    if (typeof params == "function") {
        callback = params;
        params = {};
    }
    var filepath = item.path;
    var values = item.values || {};
    var url = params["_id"] && params["_relation"] ? ("/" + modelName + "/" + params["_id"] + "/" + params["_relation"]) : "/file";
    var isPut = (!params["_relation"]) && params["_id"];
    var fileUrl = this.baseurl + url + (isPut ? ("/" + params["_id"]) : "");
    var filename = filepath.substr(filepath.lastIndexOf("/") + 1, filepath.length);
    if (!values["filename"]) values["filename"] = filename;
    var ajaxConfig = {
        url: fileUrl,
        method: isPut ? "PUT" : "POST",
        data: {
            values: values,
            files: {
                file: filepath
            }
        }
    }
    ajaxConfig["headers"] = {};
    for (var header in this.headers) {
        if (header == "Content-Type") continue;
        ajaxConfig["headers"][header] = this.headers[header];
    }
    api.ajax(ajaxConfig, function (ret, err) {
        if (ret && ret.id && !err) {
            var newobj = {};
            if (isFilter) {
                newobj["id"] = ret["id"];
                newobj["name"] = ret["name"];
                newobj["url"] = ret["url"];
                callback(null, newobj)
            } else {
                callback(null, ret);
            }
        } else {
            callback(ret || err, null)
        }
    });
}

Resource.prototype.Factory = function (modelName) {
    var self = this;
    var route = new Route(modelName, self.baseurl);
    var actions = copy(this.defaultactions);
    var resourceFactory = new Object();
    Object.keys(actions).forEach(function (name) {
        if (modelName != "user" && ["login", "logout", "verify", "reset"].indexOf(name) != -1) { return; }
        resourceFactory[name] = function (a1, a2, a3) {
            var action = copy(actions[name]);
            var params = {}, data, callback;
            var hasBody = /^(POST|PUT|PATCH)$/i.test(action.method);
            switch (arguments.length) {
                case 3:
                    params = a1;
                    data = a2;
                    callback = a3;
                    break;
                case 2:
                    if (hasBody)
                        data = a1;
                    else
                        params = a1;
                    callback = a2;
                    break;
                case 1:
                    if (isFunction(a1))
                        callback = a1;
                    else if (hasBody)
                        data = a1;
                    else
                        params = a1;
                    break;
                case 0:
                    break;
                default:
                    throw new Error("参数最多为3个");
            }
            if (hasBody && name != "logout") {
                var fileCount = 0;
                Object.keys(data).forEach(function (key) {
                    var item = data[key];
                    if (item && item.isFile) {
                        var isFilter = true;
                        if (modelName == "file" || item.isFileClass) {
                            isFilter = false;
                        }
                        fileCount++;
                        self.upload(modelName, isFilter, item, (modelName == "file" ? params : {}), function (err, returnData) {
                            if (err) {
                                return callback(null, err);
                            } else {
                                if (!isFilter)
                                    return callback(returnData, null);
                                data[key] = returnData;
                                fileCount--;
                                if (fileCount == 0) {
                                    next();
                                }
                            }
                        })
                    }
                });
                if (fileCount == 0) {
                    next();
                }
            } else {
                next();
            }
            function next() {
                var httpConfig = {};
                httpConfig["headers"] = {};
                for (var header in self.headers) {
                    httpConfig["headers"][header] = self.headers[header];
                }
                if (name === "logout" && !httpConfig["headers"]["authorization"]) {
                    return callback({ status: 0, msg: "未设置authorization参数,无法注销!" }, null);
                }
                if (hasBody) {
                    httpConfig.data = {
                        body: JSON.stringify(data)
                    };
                }

                if (params && (name == "save") && params["_id"] && (!params["_relation"]) && (!params["_relationid"])) {
                    action.method = "PUT";
                }
                if (params && (name == "save") && params["_id"] && params["_relation"] && params["_relationid"]) {
                    action.method = "PUT";
                }
                for (var key in action) {
                    if (key != 'params' && key != "alias") {
                        httpConfig[key] = copy(action[key]);
                    }
                }

                var curparams = {};
                action.params = action.params || [];
                for (var k = 0, len = action.params.length; k < len; k++) {
                    var tempkey = action.params[k];
                    if (params[tempkey]) {
                        curparams[tempkey] = copy(params[tempkey]);
                    }
                }
                if (["login", "logout", "count", "exists", "verify", "reset", "findOne"].indexOf(name) != -1) {
                    curparams["_custom"] = action.alias || name;
                }
                route.setUrlParams(httpConfig, curparams);
                httpConfig.cache = true;
                console.log(httpConfig.method + "\t" + httpConfig.url);
                api.ajax(httpConfig, function (ret, err) {
                    return callback(ret, err);
                })
            }
        };
    });
    return resourceFactory;
};

function Route(template, baseurl) {
    this.template = template;
    this.baseurl = baseurl;
}

Route.prototype = {
    setUrlParams: function (config, params) {
        var url = "/:_class/:_id/:_relation/:_custom/:_relationid";
        url = url.replace(":_class", this.template);
        var parArr = [];
        Object.keys(params).forEach(function (ckey) {
            if (ckey.charAt(0) == '_') {
                url = url.replace(":" + ckey, params[ckey]);
                delete params[ckey];
            } else {
                if (ckey == "filter") {
                    parArr.push(ckey + "=" + JSON.stringify(params[ckey]));
                }
            }
        });
        url = url.replace(/:[^/]+/ig, '/');
        if (parArr.length > 0) {
            url += ("?" + parArr.join("&"));
        }
        url = url.replace(/\/+/g, '/');
        config.url = this.baseurl + url;
    }
};
//-----------------------------------------------------------
/*
 * 向控制台输出日志
 * @param  data data  要输出的日志
 * @return string
 */
function log(data) {
	if ( typeof data == 'string') {
		console.log(data);
	} else {
		console.log(JSON.stringify(data));
	}
}
// 获取当前的时间，拼接成2015-11-09这样的格式，主要用于对图片进行时间分类
function getNowFormatDate() {
	var date = new Date();
	var seperator1 = "-";
	var year = date.getFullYear();
	var month = date.getMonth() + 1;
	var strDate = date.getDate();
	if (month >= 1 && month <= 9) {
		month = "0" + month;
	}
	if (strDate >= 0 && strDate <= 9) {
		strDate = "0" + strDate;
	}
	var currentdate = year + seperator1 + month + seperator1 + strDate
	return currentdate;
}

// 获取文件拓展名
function getExt(fileName) {
	return fileName.substring(fileName.lastIndexOf('.') + 1);
}

// 图片压缩
// imgsrc：源图片的路径
// quality：图片压缩质量，一般建议0.5
// scale：图片压缩比例，也是建议0.5
// ext：源图片拓展名
// callback：转换成功之后回调函数
function imgCompress(imgsrc, quality, scale, ext, callback) {
	var imageFilter = api.require('imageFilter');
	
	//计算压缩后的图片尺寸
	var ww, hh = 960;
	imageFilter.getAttr({
		path: imgsrc
	}, function(ret, err) {
		var width = ret.width;
		var height = ret.height;
		if(width <= 960) {
			ww = width;
			hh = height;
		} else {
			ww = 960;
            hh = parseInt(height * 960 / width);
            console.log('压缩后的图片尺寸为' + ww + ',' + hh);
		}
		// 压缩文件的保存目录
		var savePath = api.cacheDir + "/" + getNowFormatDate() + "/";
		// 压缩文件生成的随机文件名称
		var savename = NewGuid() + "." + ext;
		imageFilter.compress({
			img : imgsrc,
			quality : quality,
			scale : quality,
			size: {
				w: ww,
				h: hh
			},
			save : {
				album : false,
				imgPath : savePath,
				imgName : savename
			}
		}, function(ret, err) {
			if (ret) {
				callback(savePath + savename);
			} else {
				alert(JSON.stringify(err));
			}
		});
	}); 

}
// 生成guid,主要用于生成随机文件名
function NewGuid() {
        function S4() {
                return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
        }

        return (S4() + S4() + "-" + S4() + "-" + S4() + "-" + S4() + "-" + S4() + S4() + S4());
}
// 判断用户是否登录，但这种判断方式不太准确
// 由于使用Cookie 跟踪用户，但APICLOUD对Cookie支持不好，所以我也没啥好办法
function isLogin(){
	var user_uid = parseInt($api.getStorage('user_info_id'));
	if(user_uid){
		return true;
	}else{
		return false;
	}
}

// 打开登录页
function openLogin(){
	if(isLogin()){
		return false;
	}
	var delay = 0;
    if(api.systemType != 'ios'){
        delay = 300;
    }
	api.openWin({
	    name: 'login_win',
	    url: 'widget://html/login_win.html',
	    bounces:false,
        delay: delay,
        slidBackEnabled:true,
        vScrollBarEnabled:false,
        animation: {
		    type:"movein",                //动画类型（详见动画类型常量）
		    subType:"from_right",       //动画子类型（详见动画子类型常量）
		    duration:300                //动画过渡时间，默认300毫秒
		},
	});
}

function openLogin(name, frm_name, callback_name){
	if(isLogin()){
		return false;
	}
	var delay = 0;
    if(api.systemType != 'ios'){
        delay = 300;
    }
	api.openWin({
	    name: 'login_win',
	    url: 'widget://html/login_win.html',
	    bounces:false,
        delay: delay,
        slidBackEnabled:true,
        vScrollBarEnabled:false,
        animation: {
		    type:"movein",                //动画类型（详见动画类型常量）
		    subType:"from_right",       //动画子类型（详见动画子类型常量）
		    duration:300                //动画过渡时间，默认300毫秒
		},
	    pageParam: {
	    	name: name,
	    	frm_name: frm_name,
	    	callback_name: callback_name
	    }
	});
	
}

function showLoading(time){
    var body = $api.dom("body");
    var loading = $api.byId("loading");
    if(!loading){
        $api.append(body,'<div class="aui-loading" id="loading"></div>');
        if(time && time>0){
            //如果有时间传进来就定时隐藏
            $api.remove(loading);
        }
    }
}

function hideLoading(){
    var loading = $api.byId("loading");
    $api.remove(loading);
    
}

function getDateDiff(stringTime) {
	var result = ''
	var dateTimeStamp = Date.parse(new Date(stringTime));
	//dateTimeStamp = dateTimeStamp / 1000;
	//JavaScript函数：
	var minute = 1000 * 60;
	var hour = minute * 60;
	var day = hour * 24;
	var halfamonth = day * 15;
	var month = day * 30;
	
	var now = new Date().getTime();
	var diffValue = now - dateTimeStamp;
	if (diffValue < 0) {
		//若日期不符则弹出窗口告之
		//alert("结束日期不能小于开始日期！");
	}
	var monthC = diffValue / month;
	var weekC = diffValue / (7 * day);
	var dayC = diffValue / day;
	var hourC = diffValue / hour;
	var minC = diffValue / minute;
	if (monthC >= 1) {
		result = "发表于" + parseInt(monthC) + "个月前";
	} else if (weekC >= 1) {
		result = "发表于" + parseInt(weekC) + "周前";
	} else if (dayC >= 1) {
		result = "发表于" + parseInt(dayC) + "天前";
	} else if (hourC >= 1) {
		result = "发表于" + parseInt(hourC) + "个小时前";
	} else if (minC >= 1) {
		result = "发表于" + parseInt(minC) + "分钟前";
	} else
		result = "刚刚发表";
	return result;
}
/*
 * 返回user_info对象 直接用ret.**调用
 */
function getUserInfoByUserId(user_id, callback) {
	
	var model = api.require('model');
	model.config({
	    appKey: 'BA801BA8-6B65-A068-2FB5-E7AA4B342C30',
	    host: 'https://d.apicloud.com'
	});
	
	var query = api.require('query');
	query.createQuery(function (ret, err) {
		var queryId = ret.qid;
	    if (ret && ret.qid) {
	        query.whereEqual({
	            qid: ret.qid,
	            column: 'user_id',
	            value: user_id
	        });
	        query.limit({qid:ret.qid, value:1});
	        
			model.findAll({
	            class: "user_info",
	            qid: queryId
	        }, function(ret, err) {
	            if (ret) {
	                callback(ret[0]);
	            } else {
	            	callback('', err);
	            }
	
	        });
	    }
	});
}

//==========================================

function factory(modelName){
	var appId = 'A6925485990681';
	var key = '1CD97A8D-B550-5329-3A1D-8CEBF4858AA8';
	var client = new Resource(appId, key);
    if($api.getStorage('token')){
        client.setHeaders("authorization",$api.getStorage('token'));
    }
	return client.Factory(modelName);
}
Date.prototype.Format = function (fmt) { //author: meizz 
	var o = {	
	"M+": this.getMonth() + 1, //月份 
	"d+": this.getDate(), //日 
	"h+": this.getHours(), //小时 
	"m+": this.getMinutes(), //分 
	"s+": this.getSeconds(), //秒 
	"q+": Math.floor((this.getMonth() + 3) / 3), //季度 
	"S": this.getMilliseconds() //毫秒 
	};
	if (/(y+)/.test(fmt))
	fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
	for (var k in o)
	if (new RegExp("(" + k + ")").test(fmt))
	fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
	return fmt;
	}